/*
  ConvertToGradient.jsx for Adobe Illustrator
  Description: Convert a flat process color into a matching gradient
  Date: August, 2018
  Author: Sergey Osokin, email: hi@sergosokin.ru
  Based on script by Saurabh Sharma (https://tutsplus.com/authors/saurabh-sharma), 2010
  What's new: The script now works with the RGB and CMYK document profile, Spot & Gray colors. 
              Processes compound paths and groups of items

  Installation: https://github.com/creold/illustrator-scripts#how-to-run-scripts

  Versions:
  0.1 Initial version
  0.1.1 Performance optimization

  Donate (optional):
  If you find this script helpful, you can buy me a coffee
  - via PayPal http://www.paypal.me/osokin/usd
  - via QIWI https://qiwi.com/n/OSOKIN​
  - via YooMoney https://yoomoney.ru/to/410011149615582​

  NOTICE:
  Tested with Adobe Illustrator CC 2018-2021 (Mac), 2021 (Win).
  This script is provided "as is" without warranty of any kind.
  Free to use, not for sale.

  Released under the MIT license.
  http://opensource.org/licenses/mit-license.php

  Check other author's scripts: https://github.com/creold
*/

//@target illustrator

// Global variables
var SCRIPT_NAME = 'ConvertToGradient',
    SCRIPT_VERSION = 'v.0.1.1',
    DLG_OPACITY = .96; // UI window opacity. Range 0-1

var fillBad = 0,
    channel = new Array();

function main() {
  if (!documents.length) {
    alert('Error\nOpen a document and try again');
    return;
  }
  
  if (!selection.length || selection.typename == 'TextRange') {
    alert('Error\nPlease select atleast one object');
    return;
  }

  var doc = app.activeDocument,
      maxValue = 0,
      shiftValue = 0,
      angleValue = 0,
      gShiftEnd = 0;

  // Get initial data
  if (isRgbDoc()) {
    maxValue = 255;
    channel = ['red', 'green', 'blue'];
  } else {
    maxValue = 100;
    channel = ['cyan', 'magenta', 'yellow', 'black'];
  }

  // Main Window
  var dialog = new Window('dialog', SCRIPT_NAME + ' ' + SCRIPT_VERSION);
      dialog.preferredSize.width = 174;
      dialog.orientation = 'column';
      dialog.alignChildren = ['fill', 'fill'];
      dialog.opacity = DLG_OPACITY;

  // Value fields
  var shiftPanel = dialog.add('panel', undefined, 'Gradient Shift');
      shiftPanel.alignChildren = ['fill', 'fill'];
  var gShift = shiftPanel.add('edittext', undefined, '10');
      gShift.active = true;
  var anglePanel = dialog.add('panel', undefined, 'Gradient Angle');
      anglePanel.alignChildren = ['fill', 'fill'];
  var gAngle = anglePanel.add('edittext', undefined, '0');

  // Buttons
  var btns = dialog.add('group');
      btns.alignChildren = ['fill', 'fill'];
      btns.orientation = 'column';
  var cancel = btns.add('button', undefined, 'Cancel', { name: 'cancel' });
      cancel.helpTip = 'Press Esc to Close';
  var ok = btns.add('button', undefined, 'OK', { name: 'ok' });
      ok.helpTip = 'Press Enter to Run';

  ok.onClick = function () {
    if (isNaN(Number(gShift.text))) {
      alert('Gradient Shift\nPlease enter a numeric value.');
      return;
    } else if (gShift.text === null) {
      return;
    } else {
      shiftValue = Math.round(Number(gShift.text));
      if (shiftValue <= 0) shiftValue = 0;
      if (shiftValue >= maxValue) shiftValue = maxValue;
      gShiftEnd = maxValue - shiftValue;
    }

    if (isNaN(Number(gAngle.text))) {
      alert('Gradient Angle: \nPlease enter a numeric value.');
      return;
    } else if (gAngle.text === null) {
      return;
    } else {
      angleValue = Number(gAngle.text);
    }

    // Start conversion
    for (var i = 0, selLen = selection.length; i < selLen; i++) {
      convertToGradient(selection[i], shiftValue, angleValue, maxValue, gShiftEnd);
    }
    if (fillBad > 0) {
      alert('Fill an ' + fillBad + ' objects with flat color ' +
            '\nAny objects containing gradients, patterns, ' + 
            'global colors or empty fills will be omitted');
    }
    dialog.close();
  }

  cancel.onClick = function () { dialog.close(); }

  dialog.center();
  dialog.show();
}

// Search items in selection
function convertToGradient(obj, shift, angle, max, shiftEnd) {
  try {
    switch (obj.typename) {
      case 'GroupItem':
        for (var j = 0, piLen = obj.pageItems.length; j < piLen; j++) {
          convertToGradient(obj.pageItems[j], shift, angle, max, shiftEnd);
        }
        break;
      case 'PathItem':
        if (obj.filled == true && chkFillType(obj) == true) {
          applyGradient(obj, shift, angle, max, shiftEnd);
        } else {
          fillBad++;
        }
        break;
      case 'CompoundPathItem':
        if (obj.pathItems[0].filled == true && chkFillType(obj.pathItems[0]) == true) {
          applyGradient(obj.pathItems[0], shift, angle, max, shiftEnd);
        } else {
          fillBad++;
        }
        break;
      default:
        break;
    }
  } catch (e) {}
}

// Apply gradient to items
function applyGradient(obj, shift, angle, max, shiftEnd) {
  var currentColor = (obj.fillColor.typename == 'SpotColor') ? obj.fillColor.spot.color : obj.fillColor;
  var startColor = (isRgbDoc()) ? new RGBColor() : new CMYKColor();
  var endColor = (isRgbDoc()) ? new RGBColor() : new CMYKColor();

  // For Grayscale mode color is set individually
  if (currentColor.typename == 'GrayColor') {
    var startColor = new GrayColor();
    var endColor = new GrayColor();
    var grayColor = Math.round(currentColor.gray);

    if (grayColor < shift) startColor.gray = 0;
    else startColor.gray = grayColor - shift;

    if (grayColor > (100 - shift)) endColor.gray = 100;
    else endColor.gray = grayColor + shift;
  }

  //Set color for RGB || CMYK channels
  for (var j = 0, len = channel.length; j < len; j++) {
    var channelName = channel[j];
    var originColor = Math.round(currentColor[channelName]);
    
    if (originColor < shift) startColor[channelName] = 0;
    else startColor[channelName] = originColor - shift;

    if (originColor > shiftEnd) endColor[channelName] = max;
    else endColor[channelName] = originColor + shift;
  }

  // Create a new gradient
  var newGradient = activeDocument.gradients.add();
  newGradient.type = GradientType.LINEAR;

  // Modify the first gradient stop
  newGradient.gradientStops[0].rampPoint = 0;
  newGradient.gradientStops[0].midPoint = 50;
  newGradient.gradientStops[0].color = startColor;

  // Modify the last gradient stop
  newGradient.gradientStops[1].rampPoint = 100;
  newGradient.gradientStops[1].color = endColor;

  // Construct an Illustrator.GradientColor object
  var colorOfGradient = new GradientColor();
  colorOfGradient.gradient = newGradient;

  // Apply new gradient to current path item
  obj.fillColor = colorOfGradient;
  obj.rotate(angle, false, false, true, false, Transformation.CENTER);
}

function isRgbDoc() {
  return activeDocument.documentColorSpace == DocumentColorSpace.RGB;
}

function chkFillType(obj) {
  if (obj.fillColor.typename == 'RGBColor' ||
      obj.fillColor.typename == 'CMYKColor' ||
      obj.fillColor.typename == 'GrayColor' ||
      obj.fillColor.typename == 'SpotColor')
      return true;
}

function showError(err) {
  alert(err + ': on line ' + err.line, 'Script Error', true);
}

// Run script
try {
  main();
} catch (e) {
  // showError(e);
}